@@ -5,16 +5,24 @@ import os
 import weakref
 from contextlib import contextmanager
 from copy import deepcopy
+import warnings
 
-import confect.parser
-from confect.error import (ConfGroupExistsError, FrozenConfGroupError,
-                           FrozenConfPropError, UnknownConfError)
+
+import confect.prop_type
+from confect.error import (
+    ConfGroupExistsError,
+    FrozenConfGroupError,
+    FrozenConfPropError,
+    UnknownConfError,
+    ParameterError,
+)
 
 logger = logging.getLogger(__name__)
 
 
 class Undefined:
-    '''Undefined value'''
+    """Undefined value"""
+
     __instance = None
     __slots__ = ()
 
@@ -28,7 +36,7 @@ class Undefined:
         return False
 
     def __repr__(self):
-        return f'<{__name__}.{type(self).__qualname__}>'
+        return f"<{__name__}.{type(self).__qualname__}>"
 
     def __deepcopy__(self, memo):
         return self.__instance
@@ -39,14 +47,14 @@ Undefined = Undefined()
 
 class ConfProperty:
 
-    __slots__ = ('_value', 'default', 'parser')
+    __slots__ = ("_value", "default", "prop_type", "desc")
 
-    def __init__(self, default=Undefined, parser=None):
-        '''Create configuration property with details
+    def __init__(self, default=Undefined, *, parser=None, desc="", prop_type=None):
+        """Create configuration property with details
 
         >>> import confect
         >>> import datetime as dt
-        >>> conf = confect.new_conf()
+        >>> conf = confect.Conf()
 
         >>> from enum import Enum
         >>> class Color(Enum):
@@ -54,27 +62,58 @@ class ConfProperty:
         ...     GREEN = 2
         ...     BLUE = 3
 
-        >>> with conf.declare_group('dummy') as cg:
-        ...     cg.a_number = 3
-        ...     cg.some_string = 'some string'
-        ...     cg.color = conf.prop(
+        >>> with conf.declare_group('db') as db:
+        ...     db.port = 3361
+        ...     db.database = 'default'
+        ...     db.host = conf.prop(
+        ...          default='10.3.14.15',
+        ...          desc='hostname of database')
+        ...     db.color = conf.prop(
         ...          default=Color.RED,
-        ...          parser=lambda s: getattr(Color, s.upper()))
+        ...          prop_type=confect.make_prop_type(
+        ...              python_type=Color,
+        ...              parser=lambda s: getattr(Color, s.upper())))
 
         Paramaters
         ----------
-        default : ValueType
+        default : Any
             default value
+        prop_type : confect.PropertyType
+            confect PropretyType for parsing the string from environment
+            variable or CLI argument into Python type.
+        desc : str
+            description
         parser : Callable[[str], ValueType]
-            parser for reading environment variable or command line
-            argument into property value
+            Deprecated.
+            parser for construct confect PropertyType automatically
 
-        '''
+        """
         self.default = default
         self._value = Undefined
-        if parser is None:
-            parser = confect.parser.of_value(default)
-        self.parser = parser
+        if parser is not None and prop_type is not None:
+            raise ParameterError(
+                "`prop_type`, `parser` can't be assigned at the same time."
+            )
+
+        if parser is not None:
+            warnings.warn(
+                "`parser` argument of ConfProperty is deprecated.", DeprecationWarning
+            )
+            prop_type = confect.prop_type.make_prop_type(type(default), parser)
+        elif prop_type is not None:
+            if not isinstance(prop_type, confect.prop_type.PropertyType):
+                raise ValueError(
+                    "`prop_type` is not instance of conefct.prop_type.PropertyType."
+                )
+            prop_type = prop_type
+        else:
+            prop_type = confect.prop_type.of_value(default)
+
+            if prop_type is None:
+                raise ValueError(f"No confect.PropertyType matched: {default!r}")
+
+        self.prop_type = prop_type
+        self.desc = desc
 
     @property
     def value(self):
@@ -87,17 +126,27 @@ class ConfProperty:
     def value(self, value):
         self._value = value
 
+    def click_callback(self, ctx, param, value):
+        if param.default != value:
+            self._value = value
+
     def __repr__(self):
-        return (f'<{__name__}.{type(self).__qualname__} '
-                f'default={self.default!r} value={self._value!r} '
-                f'parser={self.parser}>')
+        return (
+            f"<{__name__}.{type(self).__qualname__} "
+            f"default={self.default!r} value={self._value!r} "
+            f"prop_type={self.prop_type}>"
+            f"desc={self.desc}"
+        )
+
+    def __str__(self):
+        return f"{self.desc} [default: {self.default!s}, value: {self.value!s}]"
 
 
 class Conf:
-    '''Configuration
+    """Configuration
 
     >>> import confect
-    >>> conf = confect.new_conf()
+    >>> conf = confect.Conf()
 
     Declare new configuration properties with
     ``Conf.declare_group(group_name)``
@@ -117,19 +166,21 @@ class Conf:
     Configuration properties can only be changed globally by loading configuration file through ``Conf.load_file()`` and ``Conf.load_module()``.
     And it can be changed locally in the context created by `Conf.mutate_locally()`.
 
-    '''  # noqa
-    __slots__ = ('_is_setting_imported',
-                 '_is_frozen',
-                 '_conf_depot',
-                 '_conf_groups',
-                 '__weakref__',
-                 )
+    """  # noqa
+
+    __slots__ = (
+        "_is_setting_imported",
+        "_is_frozen",
+        "_conf_depot",
+        "_conf_groups",
+        "__weakref__",
+    )
 
     def __init__(self):
-        '''Create a new confect.Conf object
+        """Create a new confect.Conf object
 
         >>> import confect
-        >>> conf = confect.new_conf()
+        >>> conf = confect.Conf()
 
         Declare new configuration properties with
         ``Conf.declare_group(group_name)``
@@ -141,16 +192,17 @@ class Conf:
         >>> conf.dummy.opt1
         3
 
-        '''
+        """
 
         from confect.conf_depot import ConfDepot
+
         self._is_setting_imported = False
         self._is_frozen = True
         self._conf_depot = ConfDepot()
         self._conf_groups = {}
 
     def declare_group(self, name, **default_properties):
-        '''Add new configuration group and all property names with default values
+        """Add new configuration group and all property names with default values
 
         >>> conf = Conf()
 
@@ -169,10 +221,9 @@ class Conf:
         ...                    str_prop='some string')
         >>> conf.dummy.num_prop
         3
-        '''
+        """
         if name in self._conf_groups:
-            raise ConfGroupExistsError(
-                f'configuration group {name!r} already exists')
+            raise ConfGroupExistsError(f"configuration group {name!r} already exists")
 
         with self.mutate_globally():
             group = ConfGroup(self, name)
@@ -192,7 +243,7 @@ class Conf:
 
     @contextmanager
     def mutate_locally(self):
-        '''Return a context manager that makes this Conf mutable temporarily.
+        """Return a context manager that makes this Conf mutable temporarily.
 
         All configuration properties will be restored upon completion of the block.
 
@@ -209,7 +260,7 @@ class Conf:
         >>> print(conf.yummy.name)
         fish
 
-        '''  # noqa
+        """  # noqa
         conf_groups_backup = self._backup()
         with self.mutate_globally():
             yield
@@ -224,6 +275,7 @@ class Conf:
     @contextmanager
     def _confect_c_ctx(self):
         import confect
+
         confect.c = self._conf_depot
         yield
         del confect.c
@@ -233,8 +285,7 @@ class Conf:
 
     def __getitem__(self, group_name):
         if group_name not in self._conf_groups:
-            raise UnknownConfError(
-                f'Unknown configuration group {group_name!r}')
+            raise UnknownConfError(f"Unknown configuration group {group_name!r}")
 
         conf_group = self._conf_groups[group_name]
 
@@ -250,9 +301,9 @@ class Conf:
 
     def __setitem__(self, group_name, group):
         raise FrozenConfGroupError(
-            'Configuration groups are frozen. '
-            'Call `confect.declare_group()` for '
-            'registering new configuration group.'
+            "Configuration groups are frozen. "
+            "Call `confect.declare_group()` for "
+            "registering new configuration group."
         )
 
     def __setattr__(self, name, value):
@@ -278,7 +329,7 @@ class Conf:
         return new_self
 
     def load_file(self, path):
-        '''Load python configuration file through file path.
+        """Load python configuration file through file path.
 
         All configuration groups and properties should be added through ``Conf.declare_group()`` in your source code.
         Otherwise, it won't be accessable even if it is in configuration file.
@@ -293,17 +344,18 @@ class Conf:
             from confect import c
             c.yammy.kind = 'seafood'
             c.yammy.name = 'fish'
-        '''  # noqa
+        """  # noqa
         from pathlib import Path
+
         if not isinstance(path, Path):
             path = Path(path)
 
         with self.mutate_globally():
             with self._confect_c_ctx():
-                exec(path.open('r').read())
+                exec(path.open("r").read())
 
     def load_module(self, module_name):
-        '''Load python configuration file through import.
+        """Load python configuration file through import.
 
         The module should be importable either through PYTHONPATH
         or was install as a package.
@@ -323,13 +375,13 @@ class Conf:
             c.yammy.kind = 'seafood'
             c.yammy.name = 'fish'
 
-        '''  # noqa
+        """  # noqa
         with self.mutate_globally():
             with self._confect_c_ctx():
                 importlib.import_module(module_name)
 
     def load_envvars(self, prefix):
-        '''Load python configuration from environment variables
+        """Load python configuration from environment variables
 
         This function automatically searches environment variable in
         ``<prefix>__<group>__<prop>`` format. Be aware of that all of these
@@ -339,7 +391,7 @@ class Conf:
         property to the parsed value of ``proj_X__cache__expire_time``
         environment variable.
 
-        >> conf = confect.new_conf()
+        >> conf = confect.Conf()
         >> conf.load_envvars('proj_X')  # doctest: +SKIP
 
         Parameters
@@ -347,29 +399,62 @@ class Conf:
         prefix : str
             prefix of environment variables
 
-        '''
-        prefix = prefix + '__'
+        """
+        prefix = prefix + "__"
         with self.mutate_globally():
             for name, value in os.environ.items():
                 if name.startswith(prefix):
-                    _, group, prop = name.split('__')
+                    _, group, prop = name.split("__")
                     value = self.parse_prop(group, prop, value)
                     self._conf_depot[group][prop] = value
 
     def parse_prop(self, group, prop, string):
         return self[group].parse_prop(prop, string)
 
+    def get_prop(self, group, prop):
+        return self[group].get_prop(prop)
+
     @fnt.wraps(ConfProperty.__init__)
     def prop(self, *args, **kwargs):
         return ConfProperty(*args, **kwargs)
 
+    def _iter_props(self):
+        for group_name, group in self._conf_groups.items():
+            for prop_name, prop in group._properties.items():
+                yield group_name, prop_name, prop
+
+    def click_options(self, cmd_func):
+        """Attaches all configurations to the command in
+        the `--<group>-<prop>` form."""
+        import click
+
+        props = reversed(list(self._iter_props()))
+
+        for group_name, prop_name, prop in props:
+            if prop.prop_type is None:
+                continue
+
+            cmd_func = click.option(
+                f"--{group_name}-{prop_name}",
+                default=prop.default,
+                callback=prop.click_callback,
+                expose_value=False,
+                type=prop.prop_type.click_param_type,
+                help=prop.desc,
+                show_default=True,
+            )(cmd_func)
+
+        return cmd_func
+
     def __repr__(self):
-        return (f'<{__name__}.{type(self).__qualname__} '
-                f'groups={list(self._conf_groups.keys())}>')
+        return (
+            f"<{__name__}.{type(self).__qualname__} "
+            f"groups={list(self._conf_groups.keys())}>"
+        )
 
 
 class ConfGroupPropertySetter:
-    __slots__ = ('_conf_group',)
+    __slots__ = ("_conf_group",)
 
     def __init__(self, conf_group):
         self._conf_group = conf_group
@@ -384,9 +469,7 @@ class ConfGroupPropertySetter:
             self[property_name] = value
 
     def __getitem__(self, property_name):
-        return self._conf_group._properties.setdefault(
-            property_name,
-            ConfProperty())
+        return self._conf_group._properties.setdefault(property_name, ConfProperty())
 
     def __setitem__(self, property_name, default):
         if isinstance(default, ConfProperty):
@@ -402,7 +485,7 @@ class ConfGroupPropertySetter:
 
 
 class ConfGroup:
-    __slots__ = ('_conf', '_name', '_properties')
+    __slots__ = ("_conf", "_name", "_properties")
 
     def __init__(self, conf: Conf, name: str):
         self._conf = weakref.proxy(conf)
@@ -421,20 +504,21 @@ class ConfGroup:
     def __getitem__(self, property_name):
         if property_name not in self._properties:
             raise UnknownConfError(
-                f'Unknown {property_name!r} property in '
-                f'configuration group {self._name!r}')
+                f"Unknown {property_name!r} property in "
+                f"configuration group {self._name!r}"
+            )
 
         return self._properties[property_name].value
 
     def __setitem__(self, property_name, value):
         if self._conf._is_frozen:
             raise FrozenConfPropError(
-                'Configuration properties are frozen.\n'
-                'Configuration properties can only be changed globally by '
-                'loading configuration file through '
-                '``Conf.load_file()`` and ``Conf.load_module()``.\n'
-                'And it can be changed locally in the context '
-                'created by `Conf.mutate_locally()`.'
+                "Configuration properties are frozen.\n"
+                "Configuration properties can only be changed globally by "
+                "loading configuration file through "
+                "``Conf.load_file()`` and ``Conf.load_module()``.\n"
+                "And it can be changed locally in the context "
+                "created by `Conf.mutate_locally()`."
             )
         else:
             self._properties[property_name].value = value
@@ -459,9 +543,17 @@ class ConfGroup:
         new_self._properties = deepcopy(self._properties)
         return new_self
 
+    def get_prop(self, prop):
+        return self._properties[prop]
+
     def parse_prop(self, prop, string):
-        return self._properties[prop].parser(string)
+        return self.get_prop(prop).prop_type.parse(string)
+
+    def as_dict(self):
+        return {name: prob.value for name, prob in self._properties.items()}
 
     def __repr__(self):
-        return (f'<{__name__}.{type(self).__qualname__} '
-                f'{self._name} properties={list(self._properties.keys())}>')
+        return (
+            f"<{__name__}.{type(self).__qualname__} "
+            f"{self._name} properties={list(self._properties.keys())}>"
+        )
